@@ -30,12 +30,13 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
 
 from gettext import gettext as _
 import apt
+import fnmatch
 import logging
 import itertools
-import platform
 import os
 import random
 import glob
+import re
 
 from gi.repository import Gio
 
@@ -104,9 +105,9 @@ class UpdateGroup(UpdateItem):
             # This shouldn't really happen. If we land here often, it's a sign
             # that something has gone wrong. Unless all pkgs are None it's not
             # a critical issue - a hit to the performance at most.
-            reason = ((not pkg or not pkg.candidate) and
-                      "Package was None or didn't have a candidate." or
-                      "%s already in _deps." % pkg.name)
+            reason = ((not pkg or not pkg.candidate)
+                      and "Package was None or didn't have a candidate."
+                      or "%s already in _deps." % pkg.name)
             logging.debug("Useless call to _add_deps. %s" % reason)
             return
         if len(self._deps) % 200 == 0 and callable(eventloop_callback):
@@ -142,8 +143,8 @@ class UpdateGroup(UpdateItem):
 
     def selection_is_inconsistent(self):
         pkgs_installing = [item for item in self.items if item.is_selected()]
-        return (len(pkgs_installing) > 0 and
-                len(pkgs_installing) < len(self.items))
+        return (len(pkgs_installing) > 0
+                and len(pkgs_installing) < len(self.items))
 
     def get_total_size(self):
         if self.to_remove:
@@ -212,13 +213,14 @@ class UpdateList():
         "Update-Manager::Never-Include-Phased-Updates")
 
     def __init__(self, parent, dist=None):
-        self.dist = dist if dist else platform.dist()[2]
+        self.dist = (dist if dist else utils.get_dist())
         self.distUpgradeWouldDelete = 0
+        self.oem_groups = []
         self.update_groups = []
         self.security_groups = []
         self.kernel_autoremove_groups = []
+        self.duplicate_groups = []
         self.ubuntu_pro_groups = []
-        self.ubuntu_pro_fake_groups = []
         self.num_updates = 0
         self.random = random.Random()
         self.ignored_phased_updates = []
@@ -344,9 +346,9 @@ class UpdateList():
         inst_ver = pkg._pkg.current_ver
         for ver in pkg._pkg.version_list:
             # discard is < than installed ver
-            if (inst_ver and
-                    apt.apt_pkg.version_compare(ver.ver_str,
-                                                inst_ver.ver_str) <= 0):
+            if (inst_ver
+                    and apt.apt_pkg.version_compare(ver.ver_str,
+                                                    inst_ver.ver_str) <= 0):
                 continue
             # check if we have a match
             for (verFileIter, index) in ver.file_list:
@@ -388,45 +390,7 @@ class UpdateList():
                 return True
         return False
 
-    def _get_linux_packages(self):
-        "Return all binary packages made by the linux-meta source package"
-        # Hard code this rather than generate from source info in cache because
-        # that might only be available if we have deb-src lines.  I think we
-        # could also generate it by iterating over all the binary package info
-        # we have, but that is costly.  These don't change often.
-        return ['linux',
-                'linux-cloud-tools-generic',
-                'linux-cloud-tools-lowlatency',
-                'linux-cloud-tools-virtual',
-                'linux-crashdump',
-                'linux-generic',
-                'linux-generic-lpae',
-                'linux-headers-generic',
-                'linux-headers-generic-lpae',
-                'linux-headers-lowlatency',
-                'linux-headers-lowlatency-lpae',
-                'linux-headers-server',
-                'linux-headers-virtual',
-                'linux-image',
-                'linux-image-extra-virtual',
-                'linux-image-generic',
-                'linux-image-generic-lpae',
-                'linux-image-lowlatency',
-                'linux-image-virtual',
-                'linux-lowlatency',
-                'linux-signed-generic',
-                'linux-signed-image-generic',
-                'linux-signed-image-lowlatency',
-                'linux-signed-lowlatency',
-                'linux-source',
-                'linux-tools-generic',
-                'linux-tools-generic-lpae',
-                'linux-tools-lowlatency',
-                'linux-tools-virtual',
-                'linux-virtual']
-
-    def _make_groups(self, cache, pkgs, eventloop_callback, to_remove=False,
-                     sensitive=True):
+    def _make_groups(self, cache, pkgs, eventloop_callback, to_remove=False, sensitive=True):
         if not pkgs:
             return []
         ungrouped_pkgs = []
@@ -436,8 +400,7 @@ class UpdateList():
         for pkg in pkgs:
             app = self._get_application_for_package(pkg)
             if app is not None:
-                app_group = UpdateApplicationGroup(pkg, app, to_remove,
-                                                   sensitive)
+                app_group = UpdateApplicationGroup(pkg, app, to_remove, sensitive)
                 app_groups.append(app_group)
             else:
                 ungrouped_pkgs.append(pkg)
@@ -458,23 +421,46 @@ class UpdateList():
         if ungrouped_pkgs:
             # Separate out system base packages. If we have already found an
             # application for all updates, don't bother.
-            meta_group = UpdateGroup(None, None, None, to_remove, sensitive)
+            linux_names = ("linux$",
+                           "linux-.*-buildinfo.*",
+                           "linux-.*-dev.*",
+                           "linux-.*-generic.*",
+                           "linux-.*-headers.*",
+                           "linux-.*-hwe.*",
+                           "linux-.*-gcp.*",
+                           "linux-.*-kernel.*",
+                           "linux-.*-lowlatency.*",
+                           "linux-.*-modules.*",
+                           "linux-.*-raspi.*",
+                           "linux-.*-tools.*",
+                           "linux-.*-virtual.*",
+                           "linux-base.*",
+                           "linux-crashdump.*",
+                           "linux-doc.*")
+            linux_regexp = re.compile("(" + "|".join(
+                ["^" + n for n in linux_names]) + ")")
+            ubuntu_base_group = UpdateGroup(None, None, None, to_remove, sensitive)
             flavor_package = utils.get_ubuntu_flavor_package(cache=cache)
-            meta_pkgs = [flavor_package, "ubuntu-standard", "ubuntu-minimal"]
-            meta_pkgs.extend(self._get_linux_packages())
-            for pkg in meta_pkgs:
-                if pkg in cache:
-                    meta_group.add(cache[pkg])
+            ubuntu_base_pkgs = [flavor_package,
+                                "ubuntu-standard",
+                                "ubuntu-minimal"]
+            for pkg in cache:
+                if linux_regexp.match(pkg.name):
+                    ubuntu_base_pkgs.append(pkg.name)
+            for pkg in ubuntu_base_pkgs:
+                if pkg in cache and not to_remove:
+                    # the check for to_remove avoids creating groups in
+                    # kernel_autoremove and duplicate package sections
+                    # which fixes LP: #1912718
+                    ubuntu_base_group.add(cache[pkg])
             for pkg in ungrouped_pkgs:
-                if meta_group.is_dependency(pkg, cache, eventloop_callback):
+                if ubuntu_base_group.is_dependency(pkg, cache,
+                                                   eventloop_callback):
                     if system_group is None:
-                        system_group = UpdateSystemGroup(cache, to_remove,
-                                                         sensitive)
+                        system_group = UpdateSystemGroup(cache, to_remove, sensitive)
                     system_group.add(pkg)
                 else:
-                    pkg_groups.append(UpdatePackageGroup(
-                        pkg, to_remove, sensitive)
-                    )
+                    pkg_groups.append(UpdatePackageGroup(pkg, to_remove, sensitive))
 
         app_groups.sort(key=lambda a: a.name.lower())
         pkg_groups.sort(key=lambda a: a.name.lower())
@@ -483,16 +469,18 @@ class UpdateList():
 
         return app_groups + pkg_groups
 
-    def update(self, cache, eventloop_callback=None, ua_security_packages=[]):
+    def update(self, cache, eventloop_callback=None, duplicate_packages=[],
+               ua_security_packages=[]):
         self.held_back = []
 
         # do the upgrade
         self.distUpgradeWouldDelete = cache.saveDistUpgrade()
 
-        pro_pkgs = []
+        oem_pkgs = []
         security_pkgs = []
         upgrade_pkgs = []
         kernel_autoremove_pkgs = []
+        duplicate_pkgs = []
 
         class FakeUbuntuProPackageCandidate:
             def __init__(self, source_name, version, size):
@@ -520,18 +508,9 @@ class UpdateList():
             def mark_delete(self):
                 pass
         fake_ua_packages = []
-        ua_packages_names = []
-        for (
-            package_name,
-            version,
-            size,
-            downloadable
-        ) in ua_security_packages:
-            if downloadable:
-                ua_packages_names.append(package_name)
-            else:
-                fake_ua_packages.append(FakeUbuntuProPackage(package_name,
-                                                             version, size))
+        for (package_name, version, size) in ua_security_packages:
+            fake_ua_packages.append(FakeUbuntuProPackage(package_name,
+                                                         version, size))
 
         # Find all upgradable packages
         for pkg in cache:
@@ -552,47 +531,51 @@ class UpdateList():
                         self.ignored_phased_updates.append(pkg)
                         continue
 
-                comes_from_pro = pkg.name in ua_packages_names
-                if comes_from_pro:
-                    pro_pkgs.append(pkg)
+                if fnmatch.fnmatch(pkg.name, 'oem-*-meta'):
+                    oem_pkgs.append(pkg)
                 elif is_security_update:
                     security_pkgs.append(pkg)
                 else:
                     upgrade_pkgs.append(pkg)
                 self.num_updates = self.num_updates + 1
 
-            if pkg.is_upgradable and not (pkg.marked_upgrade or
-                                          pkg.marked_install):
+            if pkg.is_upgradable and not (pkg.marked_upgrade
+                                          or pkg.marked_install):
                 self.held_back.append(pkg.name)
                 continue
-            if (pkg.is_auto_removable and
-                (cache.versioned_kernel_pkgs_regexp and
-                 cache.versioned_kernel_pkgs_regexp.match(pkg.name) and
-                 not cache.running_kernel_pkgs_regexp.match(pkg.name))):
+            if (pkg.is_auto_removable
+                and (cache.versioned_kernel_pkgs_regexp
+                     and cache.versioned_kernel_pkgs_regexp.match(pkg.name)
+                     and not cache.running_kernel_pkgs_regexp.match(
+                         pkg.name))):
                 kernel_autoremove_pkgs.append(pkg)
+            if (pkg.name in duplicate_packages):
+                duplicate_pkgs.append(pkg)
 
         # perform operations after the loop to not skip packages which
         # changed state due to the resolver
         for pkg in kernel_autoremove_pkgs:
             pkg.mark_delete()
+        for pkg in duplicate_pkgs:
+            pkg.mark_delete()
         for pkg in self.ignored_phased_updates:
             pkg.mark_keep()
 
-        if security_pkgs or upgrade_pkgs or pro_pkgs:
+        if security_pkgs or upgrade_pkgs:
             # There's updates available. Initiate the desktop file cache.
             pkg_names = [p.name for p in
-                         pro_pkgs
-                         + security_pkgs
-                         + upgrade_pkgs
-                         + kernel_autoremove_pkgs]
+                         security_pkgs + upgrade_pkgs
+                         + kernel_autoremove_pkgs + duplicate_pkgs]
             self._populate_desktop_cache(pkg_names)
+        self.oem_groups = self._make_groups(cache, oem_pkgs,
+                                            eventloop_callback)
         self.update_groups = self._make_groups(cache, upgrade_pkgs,
                                                eventloop_callback)
         self.security_groups = self._make_groups(cache, security_pkgs,
                                                  eventloop_callback)
         self.kernel_autoremove_groups = self._make_groups(
             cache, kernel_autoremove_pkgs, eventloop_callback, True)
-        self.ubuntu_pro_fake_groups = self._make_groups(
-            cache, fake_ua_packages, eventloop_callback, sensitive=False)
+        self.duplicate_groups = self._make_groups(
+            cache, duplicate_pkgs, eventloop_callback, True)
         self.ubuntu_pro_groups = self._make_groups(
-            cache, pro_pkgs, eventloop_callback)
+            cache, fake_ua_packages, eventloop_callback, sensitive=False)
