@@ -36,6 +36,7 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
 
 import distro_info
 import fnmatch
+import json
 import os
 import subprocess
 import sys
@@ -85,6 +86,7 @@ class UpdateManager(Gtk.Window):
         self.unity = UnitySupport()
         self.controller = None
         self.cache = None
+        self.ua_security_packages = []
         self.update_list = None
         self.meta_release = None
         self.hwe_replacement_packages = None
@@ -268,6 +270,27 @@ class UpdateManager(Gtk.Window):
                and pkg.installed:
                 self.oem_metapackages.add(pkg)
 
+    def _get_ua_security_status(self):
+        self.ua_security_packages = []
+        try:
+            p = subprocess.Popen(['pro', 'security-status', '--format=json'],
+                                 stdout=subprocess.PIPE)
+        except OSError:
+            pass
+        else:
+            while p.poll() is None:
+                while Gtk.events_pending():
+                    Gtk.main_iteration()
+                time.sleep(0.05)
+            s = json.load(p.stdout)
+            for package in s.get('packages', []):
+                status = package.get('status', '')
+                if status == 'pending_attach':
+                    name = package.get('package', '')
+                    version = package.get('version', '')
+                    size = package.get('download_size', 0)
+                    self.ua_security_packages.append((name, version, size))
+
     def _make_available_pane(self, install_count, need_reboot=False,
                              cancelled_update=False, error_occurred=False):
         self._check_hwe_support_status()
@@ -423,6 +446,8 @@ class UpdateManager(Gtk.Window):
 
         self._check_oem_metapackages()
 
+        self._get_ua_security_status()
+
         for pkgname in self.oem_metapackages:
             try:
                 if not self.cache[pkgname].is_installed:
@@ -433,7 +458,9 @@ class UpdateManager(Gtk.Window):
         self.update_list = UpdateList(self)
         try:
             self.update_list.update(self.cache, eventloop_callback=iterate,
-                                    duplicate_packages=self.duplicate_packages)
+                                    duplicate_packages=self.duplicate_packages,
+                                    ua_security_packages=self.
+                                    ua_security_packages)
         except SystemError as e:
             header = _("Could not calculate the upgrade")
             desc = _("An unresolvable problem occurred while "
