@@ -74,7 +74,8 @@ from .UnitySupport import UnitySupport
 # - screen reader does not say "Downloaded" for downloaded updates
 
 # list constants
-(LIST_NAME, LIST_UPDATE_DATA, LIST_SIZE, LIST_TOGGLE_ACTIVE) = range(4)
+(LIST_NAME, LIST_UPDATE_DATA, LIST_SIZE, LIST_TOGGLE_ACTIVE,
+ LIST_SENSITIVE) = range(5)
 
 # NetworkManager enums
 from .Core.roam import NetworkManagerHelper
@@ -261,7 +262,7 @@ class UpdatesAvailable(InternalDialog):
         changes_buffer.create_tag("versiontag", weight=Pango.Weight.BOLD)
 
         # the treeview (move into it's own code!)
-        self.store = Gtk.TreeStore(str, GObject.TYPE_PYOBJECT, str, bool)
+        self.store = Gtk.TreeStore(str, GObject.TYPE_PYOBJECT, str, bool, bool)
         self.treeview_update.set_model(None)
 
         self.image_restart.set_from_gicon(self.get_restart_icon(),
@@ -293,6 +294,8 @@ class UpdatesAvailable(InternalDialog):
         pkg_column.pack_start(pkg_toggle_renderer, False)
         pkg_column.add_attribute(pkg_toggle_renderer,
                                  'active', LIST_TOGGLE_ACTIVE)
+        pkg_column.add_attribute(pkg_toggle_renderer,
+                                 'sensitive', LIST_SENSITIVE)
         pkg_column.set_cell_data_func(pkg_toggle_renderer,
                                       self.pkg_toggle_renderer_data_func)
 
@@ -319,6 +322,8 @@ class UpdatesAvailable(InternalDialog):
         size_column = Gtk.TreeViewColumn(_("Download"), size_renderer,
                                          text=LIST_SIZE)
         size_column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        size_column.add_attribute(size_renderer,
+                                  'sensitive', LIST_SENSITIVE)
         self.treeview_update.append_column(size_column)
 
         self.treeview_update.set_headers_visible(True)
@@ -404,8 +409,11 @@ class UpdatesAvailable(InternalDialog):
     def restart_icon_renderer_data_func(self, cell_layout, renderer, model,
                                         iter, user_data):
         data = model.get_value(iter, LIST_UPDATE_DATA)
+        sensitive = model.get_value(iter, LIST_SENSITIVE)
         path = model.get_path(iter)
 
+        renderer.set_sensitive(sensitive)
+
         requires_restart = False
         if data.item and data.item.pkg:
             requires_restart = self.pkg_requires_restart(data.item.pkg)
@@ -433,6 +441,7 @@ class UpdatesAvailable(InternalDialog):
         if data.item:
             activatable = data.item.pkg.name not in self.list.held_back
             inconsistent = False
+            renderer.set_sensitive(data.item.sensitive)
         elif data.group:
             activatable = True
             inconsistent = data.group.selection_is_inconsistent()
@@ -485,8 +494,11 @@ class UpdatesAvailable(InternalDialog):
     def pkg_label_renderer_data_func(self, cell_layout, renderer, model,
                                      iter, user_data):
         data = model.get_value(iter, LIST_UPDATE_DATA)
+        sensitive = model.get_value(iter, LIST_SENSITIVE)
         name = GLib.markup_escape_text(model.get_value(iter, LIST_NAME))
 
+        renderer.set_sensitive(sensitive)
+
         if data.group:
             markup = name
         elif data.item:
@@ -992,7 +1004,7 @@ class UpdatesAvailable(InternalDialog):
             self.window_main.end_user_resizable()
         return False
 
-    def _add_header(self, name, groups):
+    def _add_header(self, name, groups, sensitive=True):
         total_size = 0
         for group in groups:
             total_size = total_size + group.get_total_size()
@@ -1000,7 +1012,8 @@ class UpdatesAvailable(InternalDialog):
             name,
             UpdateData(groups, None, None),
             humanize_size(total_size),
-            True
+            True,
+            sensitive
         ]
         return self.store.append(None, header_row)
 
@@ -1023,7 +1036,8 @@ class UpdatesAvailable(InternalDialog):
                 group.name,
                 UpdateData(None, group, group_is_item),
                 humanize_size(group.get_total_size()),
-                True
+                True,
+                group.sensitive
             ]
             group_iter = self.store.append(None, group_row)
 
@@ -1034,7 +1048,8 @@ class UpdatesAvailable(InternalDialog):
                     item.name,
                     UpdateData(None, None, item),
                     humanize_size(getattr(item.pkg.candidate, "size", 0)),
-                    True
+                    True,
+                    group.sensitive
                 ]
                 self.store.append(group_iter, item_row)
 
@@ -1077,6 +1092,9 @@ class UpdatesAvailable(InternalDialog):
                 _("Duplicate packages to be removed"),
                 self.list.duplicate_groups)
             self._add_groups(self.list.duplicate_groups)
+        if self.list.ubuntu_pro_groups:
+            self._add_header(_("Ubuntu Pro (enable in Settings…)"), self.list.ubuntu_pro_groups, sensitive=False)
+            self._add_groups(self.list.ubuntu_pro_groups)
 
         self.treeview_update.set_model(self.store)
         self.pkg_cell_area.indent_toplevel = (
